@@ -616,6 +616,12 @@ setup_relbuf (DSO *dso, debug_section *sec, int *reltype)
 	  if (rtype != R_RISCV_32)
 	    goto fail;
 	  break;
+#endif
+#if defined(EM_MCST_ELBRUS) && defined(R_E2K_32_ABS)
+	case EM_MCST_ELBRUS:
+	  if (rtype != R_E2K_32_ABS)
+		  goto fail;
+	  break;
 #endif
 	default:
 	fail:
@@ -2214,11 +2220,11 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 			     producers will use DW_FORM_strp which is
 			     more efficient.  */
 			  if (orig_len < new_len)
-			    fprintf (stderr, "Warning, not replacing comp_dir "
-				     "'%s' prefix ('%s' -> '%s') encoded as "
-				     "DW_FORM_string. "
-				     "Replacement too large.\n",
-				     comp_dir, base_dir, dest_dir);
+			    error (0, 0, "Warning, not replacing comp_dir "
+				   "'%s' prefix ('%s' -> '%s') encoded as "
+				   "DW_FORM_string. "
+				   "Replacement too large.",
+				   comp_dir, base_dir, dest_dir);
 			  else
 			    {
 			      /* Add zero (if no file part), one or more
@@ -3028,7 +3034,7 @@ static struct option optionsTable[] =
 static const char *optionsChars = "b:d:l:is:nV?";
 
 static const char *helpText =
-  "Usage: %s [OPTION...]\n"
+  "Usage: %s [OPTION...] FILE\n"
   "  -b, --base-dir=STRING           base build directory of objects\n"
   "  -d, --dest-dir=STRING           directory to rewrite base-dir into\n"
   "  -l, --list-file=STRING          file where to put list of source and \n"
@@ -3061,10 +3067,11 @@ help (const char *progname, bool error)
 }
 
 static void
-usage (const char *progname)
+usage (const char *progname, bool error)
 {
-  printf (usageText, progname);
-  exit (EXIT_SUCCESS);
+  FILE *f = error ? stderr : stdout;
+  fprintf (f, usageText, progname);
+  exit (error ? EXIT_FAILURE : EXIT_SUCCESS);
 }
 
 static DSO *
@@ -3168,8 +3175,7 @@ handle_build_id (DSO *dso, Elf_Data *build_id,
 
   if (build_id_size != 16 && build_id_size != 20)
     {
-      fprintf (stderr, "Cannot handle %Zu-byte build ID\n", build_id_size);
-      exit (1);
+      error (1, 0, "Cannot handle %Zu-byte build ID", build_id_size);
     }
 
   int i = -1;
@@ -3221,9 +3227,8 @@ handle_build_id (DSO *dso, Elf_Data *build_id,
     if (elf64_xlatetom (&x, &x, dso->ehdr.e_ident[EI_DATA]) == NULL)
       {
       bad:
-	fprintf (stderr, "Failed to compute header checksum: %s\n",
-		 elf_errmsg (elf_errno ()));
-	exit (1);
+	error (1, 0, "Failed to compute header checksum: %s",
+	       elf_errmsg (elf_errno ()));
       }
 
     x.d_type = ELF_T_PHDR;
@@ -3326,7 +3331,7 @@ main (int argc, char *argv[])
 	  break;
 
 	case 'u':
-	  usage (argv[0]);
+	  usage (argv[0], false);
 	  break;
 
 	case 'b':
@@ -3367,31 +3372,26 @@ main (int argc, char *argv[])
 
   if (optind != argc - 1)
     {
-      fprintf (stderr, "Need one FILE as input\n");
-      usage (argv[0]);
-      exit(EXIT_FAILURE);
+      error (0, 0, "Need one FILE as input");
+      usage (argv[0], true);
     }
 
   if (dest_dir != NULL)
     {
       if (base_dir == NULL)
 	{
-	  fprintf (stderr, "You must specify a base dir if you specify a dest dir\n");
-	  exit (1);
+	  error (1, 0, "You must specify a base dir if you specify a dest dir");
 	}
     }
 
   if (build_id_seed != NULL && do_build_id == 0)
     {
-      fprintf (stderr, "--build-id-seed (-s) needs --build-id (-i)\n");
-      exit (1);
+      error (1, 0, "--build-id-seed (-s) needs --build-id (-i)");
     }
 
   if (build_id_seed != NULL && strlen (build_id_seed) < 1)
     {
-      fprintf (stderr,
-	       "--build-id-seed (-s) string should be at least 1 char\n");
-      exit (1);
+      error (1, 0, "--build-id-seed (-s) string should be at least 1 char");
     }
 
   /* Ensure clean paths, users can muck with these. Also removes any
@@ -3410,14 +3410,12 @@ main (int argc, char *argv[])
 
   if (elf_version(EV_CURRENT) == EV_NONE)
     {
-      fprintf (stderr, "library out of date\n");
-      exit (1);
+      error (1, 0, "library out of date");
     }
 
   if (stat(file, &stat_buf) < 0)
     {
-      fprintf (stderr, "Failed to open input file '%s': %s\n", file, strerror(errno));
-      exit (1);
+      error (1, errno, "Failed to open input file '%s'", file);
     }
 
   /* Make sure we can read and write */
@@ -3426,8 +3424,7 @@ main (int argc, char *argv[])
   fd = open (file, O_RDWR);
   if (fd < 0)
     {
-      fprintf (stderr, "Failed to open input file '%s': %s\n", file, strerror(errno));
-      exit (1);
+      error (1, errno, "Failed to open input file '%s'", file);
     }
 
   dso = fdopen_dso (fd, file);
@@ -3440,12 +3437,22 @@ main (int argc, char *argv[])
 
       switch (dso->shdr[i].sh_type)
 	{
+	case SHT_MIPS_DWARF:
+	  /* According to the specification, all MIPS .debug_* sections are
+	     marked with ELF type SHT_MIPS_DWARF. As SHT_MIPS_DWARF is from
+	     processor-specific range, we have to check that we're actually
+	     dealing with MIPS ELF file before handling such sections.  */
+	  if (dso->ehdr.e_machine != EM_MIPS
+	      && dso->ehdr.e_machine != EM_MIPS_RS3_LE) {
+	    break;
+	  }
+	  /*@fallthrough@*/
 	case SHT_PROGBITS:
 	  name = strptr (dso, dso->ehdr.e_shstrndx, dso->shdr[i].sh_name);
 	  /* TODO: Handle stabs */
 	  if (strcmp (name, ".stab") == 0)
 	    {
-	      fprintf (stderr, "Stabs debuginfo not supported: %s\n", file);
+	      error (0, 0, "Stabs debuginfo not supported: %s", file);
 	      break;
 	    }
 	  if (!(do_build_id && no_recompute_build_id && !base_dir && !dest_dir)
@@ -3522,7 +3529,7 @@ main (int argc, char *argv[])
 	  GElf_Shdr shdr_mem;
 	  GElf_Shdr *shdr = gelf_getshdr (scn, &shdr_mem);
 	  if (shdr == NULL)
-	    error (1, 0, "Couldn't get shdr: %s\n", elf_errmsg (-1));
+	    error (1, 0, "Couldn't get shdr: %s", elf_errmsg (-1));
 
 	  /* Any sections we have changed aren't allocated sections,
 	     so we don't need to lookup any changed section sizes. */
@@ -3542,7 +3549,7 @@ main (int argc, char *argv[])
 	  GElf_Shdr shdr_mem;
 	  GElf_Shdr *shdr = gelf_getshdr (scn, &shdr_mem);
 	  if (shdr == NULL)
-	    error (1, 0, "Couldn't get shdr: %s\n", elf_errmsg (-1));
+	    error (1, 0, "Couldn't get shdr: %s", elf_errmsg (-1));
 
 	  /* A bug in elfutils before 0.169 means we have to write out
 	     all section data, even when nothing changed.
@@ -3591,7 +3598,7 @@ main (int argc, char *argv[])
 		  shdr->sh_size = sec_size;
 		  shdr->sh_offset = sec_offset;
 		  if (gelf_update_shdr (scn, shdr) == 0)
-		    error (1, 0, "Couldn't update shdr: %s\n",
+		    error (1, 0, "Couldn't update shdr: %s",
 			   elf_errmsg (-1));
 		}
 	    }
@@ -3605,15 +3612,13 @@ main (int argc, char *argv[])
 	{
 	  dso->ehdr.e_shoff = new_offset;
 	  if (gelf_update_ehdr (elf, &dso->ehdr) == 0)
-	    error (1, 0, "Couldn't update ehdr: %s\n", elf_errmsg (-1));
+	    error (1, 0, "Couldn't update ehdr: %s", elf_errmsg (-1));
 	}
     }
 
   if (elf_update (dso->elf, ELF_C_NULL) < 0)
     {
-      fprintf (stderr, "Failed to update file: %s\n",
-	       elf_errmsg (elf_errno ()));
-      exit (1);
+      error (1, 0, "Failed to update file: %s", elf_errmsg (elf_errno ()));
     }
 
   if (do_build_id && build_id != NULL)
@@ -3621,13 +3626,11 @@ main (int argc, char *argv[])
 
   if (elf_update (dso->elf, ELF_C_WRITE) < 0)
     {
-      fprintf (stderr, "Failed to write file: %s\n", elf_errmsg (elf_errno()));
-      exit (1);
+      error (1, 0, "Failed to write file: %s", elf_errmsg (elf_errno()));
     }
   if (elf_end (dso->elf) < 0)
     {
-      fprintf (stderr, "elf_end failed: %s\n", elf_errmsg (elf_errno()));
-      exit (1);
+      error (1, 0, "elf_end failed: %s", elf_errmsg (elf_errno()));
     }
   close (fd);
 
